<?php

namespace Drupal\nextjs\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the next.js integration entity edit forms.
 */
class NextjsEnvironmentForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\nextjs\Entity\NextjsEnvironmentInterface $entity */
    $entity = $this->entity;

    $form = parent::form($form, $form_state);

    $stateVisibleIfTypeHost = [
      'visible' => [
        ':input[name="type"]' => [
          'value' => 'host',
        ],
      ],
    ];

    $stateRequiredIfTypeHost = [
      'required' => [
        ':input[name="type"]' => [
          'value' => 'host',
        ],
      ],
    ];

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#description' => $this->t('Label for the Next.js Environment.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\nextjs\Entity\NextjsEnvironment::load',
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type'),
      '#options' => [
        'host' => $this->t('Host / IP-Address'),
      ],
      '#default_value' => $entity->getType(),
      '#required' => TRUE,
    ];

    $form['host_config'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Host Configuration'),
      '#states' => $stateVisibleIfTypeHost,
      '#attributes' => [
        'style' => 'display: none;',
      ],

      'host' => [
        '#type' => 'textfield',
        '#title' => $this->t('Host'),
        '#description' => $this->t('Hostname or IP-Address at which the Next.js instance is reachable.<br /><br /><strong>Note:</strong> When using DDEV to develop locally, you can use <code>host.docker.internal:3000</code> to point to your Next.js instance running on your host.'),
        '#maxlength' => 255,
        '#default_value' => $entity->getHost(),
        '#states' => $stateRequiredIfTypeHost,
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    if ($form_state->getValue('type') === 'host') {
      if (empty($form_state->getValue('host'))) {
        $form_state->setErrorByName('host', 'Host is required!');
      }

      if (!filter_var($form_state->getValue('host'), FILTER_VALIDATE_URL)) {
        $form_state->setErrorByName('host', 'Host must be a valid URL (e.g. starting with https://)');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $result = parent::save($form, $form_state);

    $entity = $this->getEntity();

    $message_arguments = ['%label' => $entity->label()];
    $logger_arguments = [
      '%label' => $entity->label(),
    ];

    switch ($result) {
      case SAVED_NEW:
        $this->messenger()->addStatus($this->t('New Next.js Environment %label has been created.', $message_arguments));
        $this->logger('nextjs')->notice('Created new Next.js Environment %label', $logger_arguments);
        break;

      case SAVED_UPDATED:
        $this->messenger()->addStatus($this->t('The Next.js Environment %label has been updated.', $message_arguments));
        $this->logger('nextjs')->notice('Updated Next.js Environment %label.', $logger_arguments);
        break;
    }

    $form_state->setRedirect('entity.nextjs_environment.collection');

    return $result;
  }

}
