<?php

namespace Drupal\nextjs\Webhook;

use Drupal\consumers\Entity\ConsumerInterface;
use Drupal\nextjs\Entity\NextjsEnvironment;
use Drupal\nextjs\Event\CacheWebhookEvent;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * WebhookDispatcher to dispatch cache webhooks.
 */
class CacheWebhookDispatcher extends WebhookDispatcher {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected ClientInterface $httpClient,
    protected EventDispatcherInterface $eventDispatcher,
    protected LoggerInterface $logger,
  ) {
    parent::__construct($httpClient, $logger);
  }

  /**
   * Dispatch a given cache webhook.
   */
  public function dispatch(ConsumerInterface $consumer, array $payload) {
    // Make sure that cache webhook is enabled for consumer.
    $webhookEnabled = $consumer->get('nextjs_cache_webhook_enabled')->getString() === "1";
    if (!$webhookEnabled) {
      $this->logger->info('Not handling cache webhook for consumer with cache webhooks disabled: @consumer', [
        '@consumer' => $consumer->getClientId(),
      ]);
      return;
    }

    // Load Next.js Environment.
    $nextjsEnvironmentId = $consumer->get('nextjs_environment')->getString();
    $nextjsEnvironment = NextjsEnvironment::load($nextjsEnvironmentId);
    if (!$nextjsEnvironment) {
      $this->logger->error('Could not send webhook, because consumer @consumer does not have a valid Next.js environment (id: "@envId") set!', [
        '@consumer' => $consumer->getClientId(),
        '@envId' => $nextjsEnvironmentId,
      ]);
      return;
    }

    $endpoint = $consumer->get('nextjs_cache_webhook_path')->getString();
    if (empty($endpoint)) {
      $this->logger->error('Could not send webhook, because consumer @consumer does not have the webhook path set!', [
        '@consumer' => $consumer->getClientId(),
      ]);
      return;
    }

    // Let other modules modify request data.
    $event = new CacheWebhookEvent($consumer, $nextjsEnvironment, $payload, $endpoint);
    $this->eventDispatcher->dispatch($event, $event::EVENT_NAME);

    // Aborted.
    if ($event->isAborted()) {
      return;
    }

    // Send request to each url.
    foreach ($event->getHosts() as $host) {
      $uri = $host['uri'];
      $fullUrl = rtrim($uri, '/ \t\n\r') . '/' . ltrim($event->getEndpoint(), '/ \t\n\r');

      $this->sendRequest(
        $fullUrl,
        $event->getPayload(),
      );
    }
  }

}
